package com.zsc.edu.dify.modules.dify.controller;

import com.zsc.edu.dify.exception.ExceptionUtil;
import com.zsc.edu.dify.framework.mybatisplus.DataPermission;
import com.zsc.edu.dify.framework.security.SecurityUtil;
import com.zsc.edu.dify.modules.dify.entity.AppEntity;
import com.zsc.edu.dify.modules.dify.service.AppEntityService;
import com.zsc.edu.dify.modules.dify.service.DifyWorkflowService;
import com.zsc.edu.dify.modules.operationLog.entity.OperationLogAnnotation;
import io.github.guoshiqiufeng.dify.workflow.dto.request.WorkflowRunRequest;
import io.github.guoshiqiufeng.dify.workflow.dto.response.WorkflowRunResponse;
import jakarta.annotation.Resource;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/api/ppt")
public class PPTController {

    @Resource
    private DifyWorkflowService difyWorkflowService;

    @Resource
    private AppEntityService appEntityService;

    /**
     * 运行从可研申报书生成科技项目PPT_工作流
     *
     * @param request
     * @return
     */
    @PostMapping("/run/{appId}")
    @OperationLogAnnotation(content = "'dify工作流'", operationType = "运行")
    public WorkflowRunResponse runWorkflow(@RequestBody WorkflowRunRequest request, @PathVariable String appId) {
        request.setUserId(SecurityUtil.getUserInfo().id.toString());
        return ExceptionUtil.difyException(() -> difyWorkflowService.run(request, appId));
    }

    /**
     * 根据appType获取应用列表
     * @return
     */
    @GetMapping("/apps")
    @DataPermission
    public List<AppEntity> getAppsByAppType(){
        return appEntityService.selectByAppType(AppEntity.AppType.PPT.getValue());
    }

}
