package com.zsc.edu.dify.modules.dify.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.zsc.edu.dify.framework.mybatisplus.DataPermission;
import com.zsc.edu.dify.modules.dify.dto.WorkflowDeptDto;
import com.zsc.edu.dify.modules.dify.entity.AppEntity;
import com.zsc.edu.dify.modules.dify.entity.WorkflowDept;
import com.zsc.edu.dify.modules.dify.service.AppEntityService;
import com.zsc.edu.dify.modules.dify.service.WorkflowDeptService;
import com.zsc.edu.dify.modules.operationLog.entity.OperationLogAnnotation;
import io.github.guoshiqiufeng.dify.server.DifyServer;
import io.github.guoshiqiufeng.dify.server.dto.response.ApiKeyResponseVO;
import io.github.guoshiqiufeng.dify.server.dto.response.AppsResponseVO;
import io.github.guoshiqiufeng.dify.server.dto.response.DatasetApiKeyResponseVO;
import jakarta.annotation.Resource;
import lombok.AllArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@AllArgsConstructor
@RestController
@RequestMapping("/api/v1/server")
public class V1ServerController {

    private DifyServer difyServer;

    private AppEntityService appEntityService;

    private final WorkflowDeptService workflowDeptService;

    /**
     * 获取应用列表
     * @param mode 模式 chat\agent-chat\completion\advanced-chat\workflow
     * @param name 应用名称，用于过滤应用列表（可选，传入空字符串时表示不过滤）
     * @return
     */
    @GetMapping("/apps")
    public List<AppEntity> getApps(String mode, String name, Integer type) {
        return appEntityService.getApps(mode, name, type);
    }

    /**
     * 获取应用详情
     * @param id 应用id
     * @return
     */
    @GetMapping("/{id}")
    public AppsResponseVO getApp(@PathVariable("id") String id) {
        return difyServer.app(id);
    }

    /**
     * 获取应用api-key
     * @param id 应用id
     * @return
     */
    @GetMapping("/api-key/{id}")
    public List<ApiKeyResponseVO> getAppApiKey(@PathVariable("id") String id) {
        return difyServer.getAppApiKey(id);
    }

    /**
     * 初始化应用api-key
     * @param id 应用id
     * @return
     */
    @PostMapping("/api-key/init/{id}")
    public List<ApiKeyResponseVO> initAppApiKey(@PathVariable("id") String id) {
        return difyServer.initAppApiKey(id);
    }

    /**
     * 获取知识库api-key
     * @return
     */
    @GetMapping("/api-key/dataset")
    public List<DatasetApiKeyResponseVO> getDatasetApiKey() {
        return difyServer.getDatasetApiKey();
    }

    /**
     * 初始化知识库api-key
     * @return
     */
    @PostMapping("/api-key/dataset/init")
    public List<DatasetApiKeyResponseVO> initDatasetApiKey() {
        return difyServer.initDatasetApiKey();
    }

    /**
     * 切换启用状态
     *
     * @param id 应用id
     * @return
     */
    @PostMapping("/app/{id}/toggle")
    @OperationLogAnnotation(content = "'dify服务启用状态'", operationType = "更新")
    public boolean enabledApp(@PathVariable String id) {
        return appEntityService.enabledApp(id);
    }

    /**
     * 获取可用应用列表
     * @return
     */
    @GetMapping("/apps/enabled")
    @DataPermission
    public List<AppEntity> getEnableApps() {
        LambdaQueryWrapper<AppEntity> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AppEntity::isEnabled, true);
        return appEntityService.list(queryWrapper);
    }

    /**
     * 根据appType获取应用列表
     * @param appType
     * @return
     */
    @GetMapping("/apps/type")
//    @DataPermission
    public List<AppEntity> getAppsByAppType(Integer appType){
        return appEntityService.selectByAppType(appType);
    }

    @PostMapping("/link")
    public ResponseEntity<String> link(@RequestBody WorkflowDeptDto workflowDeptDto) {
        List<WorkflowDept> workflowDepts = new ArrayList<>();
        for (Long deptId: workflowDeptDto.getDeptIds()) {
            workflowDepts.add(new WorkflowDept(workflowDeptDto.getWorkflowId(), deptId));
        }
        // 删除旧的关联关系
        workflowDeptService.remove(new LambdaQueryWrapper<WorkflowDept>().eq(WorkflowDept::getWorkflowId, workflowDeptDto.getWorkflowId()));
        return workflowDeptService.saveBatch(workflowDepts) ?
                ResponseEntity.ok("关联成功") :
                ResponseEntity.status(HttpStatus.BAD_REQUEST).body("关联失败");
    }
}
