package com.zsc.edu.dify.modules.dify.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zsc.edu.dify.exception.ConstraintException;
import com.zsc.edu.dify.framework.security.SecurityUtil;
import com.zsc.edu.dify.modules.dify.entity.AppEntity;
import com.zsc.edu.dify.modules.dify.entity.WorkflowDept;
import com.zsc.edu.dify.modules.dify.mapper.AppEntityMapper;
import com.zsc.edu.dify.modules.dify.repo.AppEntityRepository;
import com.zsc.edu.dify.modules.dify.repo.WorkflowDeptRepository;
import com.zsc.edu.dify.modules.dify.service.AppEntityService;
import io.github.guoshiqiufeng.dify.server.DifyServer;
import io.github.guoshiqiufeng.dify.server.dto.response.ApiKeyResponseVO;
import io.github.guoshiqiufeng.dify.server.dto.response.AppsResponseVO;
import jakarta.annotation.Resource;
import lombok.AllArgsConstructor;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * @author zhuang
 */
@AllArgsConstructor
@Service
public class AppEntityServiceImpl extends ServiceImpl<AppEntityRepository, AppEntity> implements AppEntityService {

    private final AppEntityMapper appEntityMapper;

    private final DifyServer difyServer;

    private final AppEntityRepository appEntityRepository;

    private final WorkflowDeptRepository workflowDeptRepository;


    @Override
    public List<AppEntity> getApps(String mode, String name, Integer type) {
        return this.lambdaQuery()
                .eq(StringUtils.hasText(mode), AppEntity::getMode, mode)
                .eq(StringUtils.hasText(name), AppEntity::getName, name)
                .eq(Objects.nonNull(type), AppEntity::getAppType, type)
                .list();
    }

    /**
     * 添加应用到数据库中
     * @param appsResponseList
     * @return
     */
    public List<AppsResponseVO> addApps(List<AppsResponseVO> appsResponseList) {
        for(AppsResponseVO appsResponse : appsResponseList){
            boolean isHave = baseMapper.exists(new QueryWrapper<AppEntity>().eq("id", appsResponse.getId()));
            if(!isHave){
                AppEntity appEntity = appEntityMapper.toEntity(appsResponse);
                baseMapper.insert(appEntity);
                List<ApiKeyResponseVO> apiKeyList =difyServer.getAppApiKey(appsResponse.getId());
                if (apiKeyList != null) {
                    difyServer.initAppApiKey(appsResponse.getId());
                    Optional<ApiKeyResponseVO> apiKeyResponseVO = apiKeyList.stream()
                            .filter(apiKey -> apiKey.getToken() != null)
                            .findFirst();
                    apiKeyResponseVO.ifPresent(apiKey -> appEntity.setApiKey(apiKey.getToken()));
                }
                baseMapper.updateById(appEntity);
            }else{
                baseMapper.updateById(appEntityMapper.toEntity(appsResponse));
            }

        }
        return appsResponseList;
    }

    /**
     * 启用禁用应用
     * @param id
     * @return
     */
    @Override
    public boolean enabledApp(String id) {
        AppEntity appEntity = baseMapper.selectById(id);
        if(appEntity == null){
            throw new ConstraintException("应用不存在");
        }
        appEntity.setEnabled(!appEntity.isEnabled());
        baseMapper.updateById(appEntity);
        return true;
    }

    /**
     * 根据appId获取apikey并缓存至redis
     * @param appId
     * @return
     */
    @Override
//    @Cacheable(cacheNames="dify:flow:apikey",key="#appId")
    public String getApikey(String appId){
        return appEntityRepository.selectApiKey(appId);
    }

    /**
     * 根据appType获取应用
     * @param appType
     * @return
     */
    @Override
    public List<AppEntity> selectByAppType(Integer appType) {
        Long deptId = SecurityUtil.getUserInfo().getDept().getId();
        return baseMapper.selectByAppType(deptId, appType);
    }

}